/***************************************************************************
*                 _                                _                       *
*                /_\       Chess Board Engine     /_\                      *
*                \_/      -------------------     \_/                      *
*                /_\        Based on Rules        /_\                      *
*                |_|      -------------------     |_|                      *
*               /___\      GNU GPL Version 2     /___\                     *
*                                                                          *
***************************************************************************/
/* Header Files */
#include "engine.h"

/* Window */
static WINDOW *winF;
static WINDOW *winE;

/*---------------------------------------------------------------------------*/

/* Material */
short int ebit[E] = {'1','0','0','0','1','0','1','1','1','0',0};
short int cbit[E] = { 8, 5, 2, 4, 7, 1, 3, 6,0};

/*---------------------------------------------------------------------------*/

/* Circle */
short int circle[H][A] = { 
	
{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0},
{0x00,0xDF,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0x00,0},
{0x00,0xDE,0xC3,0xB0,0xB1,0xB2,0xB3,0xB4,0xCB,0x00,0},
{0x00,0xDD,0xC2,0xAF,0xA4,0xA5,0xA6,0xB5,0xCC,0x00,0},
{0x00,0xDC,0xC1,0xAE,0xA3,0xA0,0xA7,0xB6,0xCD,0x00,0},
{0x00,0xDB,0xC0,0xAD,0xA2,0xA1,0xA8,0xB7,0xCE,0x00,0},
{0x00,0xDA,0xBF,0xAC,0xAB,0xAA,0xA9,0xB8,0xCF,0x00,0},
{0x00,0xD9,0xBE,0xBD,0xBC,0xBB,0xBA,0xB9,0xD0,0x00,0},
{0x00,0xD8,0xD7,0xD6,0xD5,0xD4,0xD3,0xD2,0xD1,0x00,0},
{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0}};

/*---------------------------------------------------------------------------*/

/* Chess Engine Array */
char cengine[H][A][E][C2] = {
			
{{{' ',9},{'8',9},{'7',9},{'6',9},{'5',9},{'4',9},{'3',9},{'2',9},{'1',9},{' ',9},{0,0}}},
{{{'a',9},{'R',5},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'r',5},{'a',9},{0,0}}},
{{{'b',9},{'N',3},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'n',3},{'b',9},{0,0}}},
{{{'c',9},{'B',4},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'b',4},{'c',9},{0,0}}},
{{{'d',9},{'Q',6},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'q',6},{'d',9},{0,0}}},
{{{'e',9},{'K',7},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'k',7},{'e',9},{0,0}}},
{{{'f',9},{'B',4},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'b',4},{'f',9},{0,0}}},
{{{'g',9},{'N',3},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'n',3},{'g',9},{0,0}}},
{{{'h',9},{'R',5},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'r',5},{'h',9},{0,0}}},
{{{' ',9},{'8',9},{'7',9},{'6',9},{'5',9},{'4',9},{'3',9},{'2',9},{'1',9},{' ',9},{0,0}}}};

/*---------------------------------------------------------------------------*/

/* Engine */
void engine(short int T1,short int T2)
{	
	/* Engine Time */
	unsigned int usecs = 100000;
	
	/* Calculation */
	char emove[20];
	short int X = 0;
	short int Y = 10;
	short int S1 = 0;
	short int S2 = 0;

	/* a - h */
	emove[1+X] = 'a';
	emove[2+X] = 'b';
	emove[3+X] = 'c';
	emove[4+X] = 'd';
	emove[5+X] = 'e';
	emove[6+X] = 'f';
	emove[7+X] = 'g';
	emove[8+X] = 'h';
	
	/* 1 - 8 */
	emove[1+Y] = '1';
	emove[2+Y] = '2';
	emove[3+Y] = '3';
	emove[4+Y] = '4';
	emove[5+Y] = '5';
	emove[6+Y] = '6';
	emove[7+Y] = '7';
	emove[8+Y] = '8';
	
	/*----------------------------------------------------------------------*/
	
	/* Move From */
	amove[0] = emove[F1+X];
	amove[1] = emove[F2+Y];
		
	/* Calculation */
	S1 = F1 + T1;
	S2 = F2 + T2;

	/* 64 Felder */
	if(((S1 >= C1) && ( S1 <= C8)) && ((S2 >= C1) && (S2 <= C8)))
	{
		/* Circle */
		if((circle[S1][S2] >= 160) && (circle[S1][S2] <= 223))
		{
			/* Move to */		
			amove[2]  = emove[X+S1];
			amove[3]  = emove[Y+S2];
								
			mvwprintw(winE,L1,14,"[%c%c/%c%c]",amove[0],amove[1],amove[2],amove[3]);
			
			/* Wait Time */
			usleep(usecs);
			
			/* Windows Refresh */
			wrefresh(winF);
			wrefresh(winE);
		}
	}
}

/*---------------------------------------------------------------------------*/

/* King */
void king(short int T1,short int T2)
{
	if(((T1 >= -1) && (T1 <= 1)) && ((T2 >= -1) && (T2 <= 1)) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* Queen */
void queen(short int T1,short int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(T1,T2);
	}
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* Turm */
void turm(short int T1,short int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* Bishop */
void bishop(short int T1,short int T2)
{
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* Springer */
void springer(short int T1,short int T2)
{
	if((((T2 == -2) || (T2 == +2)) && ((T1 == -1) || (T1 == +1))))
	{
		engine(T1,T2);
	}
	if((((T1 == -2) || (T1 == +2)) && ((T2 == -1) || (T2 == +1))))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* People B */
void people1(short int T1,short int T2)
{
	if(((T1 == 0)) && (T2 == -2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == -1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* People W */
void people2(short int T1,short int T2)
{	
	if(((T1 == 0)) && (T2 == +2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == +1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2);
	}
}

/*---------------------------------------------------------------------------*/

/* Squares */
void squares(short int *ES)
{
	/* Sort */
	short int R2, R3, R4, T1, T2;
	short int G1, G2, G3, G4, G5, G6, G7, G8;
	
	/* White */
	cengine[5][1][1][C0] = '0';
	cengine[5][1][8][C0] = '+';
	
	cengine[8][1][5][C0] = 'r';
	cengine[7][1][3][C0] = 'n';
	cengine[6][1][4][C0] = 'b';
	cengine[5][1][7][C0] = 'k';
	cengine[4][1][6][C0] = 'q';
	cengine[3][1][4][C0] = 'b';
	cengine[2][1][3][C0] = 'n';
	cengine[1][1][5][C0] = 'r'; 
			
	cengine[8][2][2][C0] = 'p'; 
	cengine[7][2][2][C0] = 'p';
	cengine[6][2][2][C0] = 'p';
	cengine[5][2][2][C0] = 'p';
	cengine[4][2][2][C0] = 'p';
	cengine[3][2][2][C0] = 'p';
	cengine[2][2][2][C0] = 'p';
	cengine[1][2][2][C0] = 'p'; 
				
	/* Black */
	cengine[5][8][1][C1] = '0';
	cengine[5][8][8][C1] = '+';
	
	cengine[8][8][5][C1] = 'R';
	cengine[7][8][3][C1] = 'N';
	cengine[6][8][4][C1] = 'B';
	cengine[5][8][7][C1] = 'K';
	cengine[4][8][6][C1] = 'Q';
	cengine[3][8][4][C1] = 'B';
	cengine[2][8][3][C1] = 'N';
	cengine[1][8][5][C1] = 'R'; 
			
	cengine[8][7][2][C1] = 'P'; 
	cengine[7][7][2][C1] = 'P';
	cengine[6][7][2][C1] = 'P';
	cengine[5][7][2][C1] = 'P';
	cengine[4][7][2][C1] = 'P';
	cengine[3][7][2][C1] = 'P';
	cengine[2][7][2][C1] = 'P';
	cengine[1][7][2][C1] = 'P'; 
	
	/*----------------------------------------------------------------------*/
		
	/* Selection Sort */
	L3++; R3 = L3;

	for(R2 = L3 + 1; R2 <= C8; R2++)
	{	
		if(cbit[R2] > cbit[R3])
		{
			R3 = R2;
		}
	}
	if(R3 != L3)
	{
		R4 = cbit[R3];
		cbit[R3] = cbit[L3];
		cbit[L3] = R4;
	}
	
	/* Result */
	G1 = cbit[0];
	G2 = cbit[1];
	G3 = cbit[2];
	G4 = cbit[3];
	G5 = cbit[4];
	G6 = cbit[5];
	G7 = cbit[6];
	G8 = cbit[7];
		
	/*------------------------------------------------------------------*/	
			
	/* Insert Sort */
	R4 = cbit[L1];
	
	for(R2 = L1; cbit[R2-1] > R4 && R2 > C0; R2--)
	{	
		cbit[R2] = cbit[R2-1];
	}	
	cbit[R2] = R4;
	
	/* Result */
	G1 = cbit[0];
	G2 = cbit[1];
	G3 = cbit[2];
	G4 = cbit[3];
	G5 = cbit[4];
	G6 = cbit[5];
	G7 = cbit[6];
	G8 = cbit[7];

	/*------------------------------------------------------------------*/
	
	/* Matrix */
	matrix[8][C0][C0][C0] = '1';
	matrix[7][C0][C0][C1] = '1';
	matrix[6][C0][C1][C0] = '2';
	matrix[5][C0][C1][C1] = '2';
	matrix[4][C1][C0][C0] = '3';
	matrix[3][C1][C0][C1] = '3';
	matrix[2][C1][C1][C0] = '4'; 
	matrix[1][C1][C1][C1] = '4';
	
	/*------------------------------------------------------------------*/
	
	/* Array */
	E1 = ES[1];
	E2 = ES[2];
	E3 = ES[3];
	E4 = ES[4];
	E5 = ES[5];
	E6 = ES[6];
	
	/*  # Squares # */
	for(F1 = C1; F1 <= C8; F1++)
	{
	for(F2 = C1; F2 <= C8; F2++)
	{
	/* Circle */
	if((circle[F1][F2] >= 160) && (circle[F1][F2] <= 223))
	{
	/* Direction */
	for(T1 = D; T1 <= C; T1++)
	{
	for(T2 = D; T2 <= C; T2++)
	{
		/* Chess Diplomatic */
		if(cengine[F1][F2][8][E6] && matrix[G8][E4][E5][E6])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][8][E6]);
			king(T1,T2);
		}
		/* Rochade */
		if(cengine[F1][F2][1][E6] && matrix[G1][E4][E5][E6])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][1][E6]);
			king(T1,T2);
		}
		/* K:  T0 | T0  */
		if(cengine[F1][F2][7][E6] && matrix[G7][E4][E5][E6])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][7][E6]);
			king(T1,T2);
		}
		/* Q:  T2 | T0  */
		if(cengine[F1][F2][6][E6] && matrix[G6][E4][E5][E6])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][6][E6]);
			queen(T1,T2);
		}	
		/* R:  T2 | T0   */
		if(cengine[F1][F2][5][E6] && matrix[G5][E4][E5][E6])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][5][E6]);
			turm(T1,T2);	
		}
		/* B:  T2 | T2   */	
		if(cengine[F1][F2][4][E6] && matrix[G4][E4][E5][E6])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][4][E6]);
			bishop(T1,T2);
		}
		/* N:  T2 | T1   */
		if(cengine[F1][F2][3][E6] && matrix[G3][E4][E5][E6])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][3][E6]);
			springer(T1,T2);
		}
		/* P:  T1 | T0   */
		if(cengine[F1][F2][2][E6] && matrix[G2][E4][E5][E6])
		{		
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][2][E6]);
			if(E6 == 0)
			{
				people2(T1,T2);
			}
			if(E6 == 1)
			{
				people1(T1,T2);
			}
		}
	}}}}}
}

/*---------------------------------------------------------------------------*/

/* Sort */
void sort(char *nmove)
{
	/* Sort */
	short int FIGU, FELD;
	short int ES[C];

	/*----------------------------------------------------------------------*/

	/* Window Box */
	winF = newwin(11,76,13,2);
	winE = newwin(9,74,14,3);
	box(winF,ACS_VLINE,ACS_HLINE);
	scrollok(winE,TRUE);
	
	/* Windows Refresh */
	touchwin(stdscr);
	refresh();
	wrefresh(winF);
	wrefresh(winE);

	/* Variables */
	L1 = -1;
	
	/*----------------------------------------------------------------------*/

	/* B / W */
	for(E1 = C0; E1 <= C1; E1++)
	{
	/* Notation */
	for(E2 = C0; E2 <= C1; E2++)
	{	
	/* Direction */
	for(E3 = C0; E3 <= C1; E3++)
	{
	wprintw(winE,"\n"); L3 = -1; L1++; L2 = 16;		
	/* Material */
	for(E4 = C0; E4 <= C1; E4++)
	{
	/* Black and White */
	for(E5 = C0; E5 <= C1; E5++)
	{
	/* Circle */
	for(E6 = C0; E6 <= C1; E6++)
	{
	/* Logic */		
	if((E1 || E2) || ((E3 || E4) || (E5 || E6)))
	{
	/* Engine Deep */	
	for(FIGU = -2, FELD = 0; FIGU < 0; FIGU += 2)
	{	
		/* Counter */
		if(L1 >= 8)
		{
			L1 = 8;
		}
		
		/* Array */
		ES[1] = E1;
		ES[2] = E2;
		ES[3] = E3;
		ES[4] = E4;
		ES[5] = E5;
		ES[6] = E6;
		
		/* Engine Info */
		FELD = FELD | FIGU;
		mvwprintw(winE,L1,0,"%+3d %d%d%d%d%d%d ", 
		FELD, ES[1]+3, ES[2]+5, ES[3], ES[4]+1, ES[5], ES[6]);	
		
		/*------------------------------------------------------------------*/	
		
		/* Function */
		squares(ES);
		
		/*------------------------------------------------------------------*/
		
		L2 += 6;

		/* Move Notation */
		mvwprintw(winE,L1,L2," %c%c%c%c ",amove[0],amove[1],amove[2],amove[3]);
	
		/* Control Sort 
		mvwprintw(winE,L1,L2," %d%d%d%d ",cbit[0],cbit[1],cbit[2],cbit[3]);*/
		
		/* Engine to Board */
		nmove[0] = amove[0];
		nmove[1] = amove[1];
		nmove[2] = amove[2];
		nmove[3] = amove[3];
		
		/* Windows Refresh */
		wrefresh(winF);
		wrefresh(winE);

	}}}}}}}}
}

/*---------------------------------------------------------------------------*/

